#include "NICMatrix.h"
#include "NICVector.h"
//#include "nic_complex.h"

#include <algorithm>

// data is stored as row1, row2, row3....

//template class NICMatrix<nic_complex<double> >;
//template class NICMatrix<nic_complex<float> >;

template <class T>
NICMatrix<T>::NICMatrix(int rows, int cols, int totalRows, 
  int totalCols, int rank, int numCpus)
: m_rows( rows ), m_columns( cols ), m_size( m_rows * m_columns ), 
  m_totalRows( totalRows ), m_totalColumns( totalCols ),
  m_totalSize( m_totalRows * m_totalColumns ), m_rank( rank ), 
  m_numCpus( numCpus )
{
  if ( m_totalRows == 0 ) m_totalRows = m_rows;
  if ( m_totalColumns == 0 ) m_totalColumns = m_columns;
  if ( m_totalSize == 0 ) m_totalSize = m_size;

  if ( m_size == 0) 
  {
    m_data = 0; 
  } else 
  {
    m_data = new T[ m_size ];
    for (int i = 0; i < m_size; ++i) 
    {
      m_data[i] = 0;
    }
  }
}

template <class T>
NICMatrix<T>::NICMatrix( const NICMatrix<T>& mat )
: m_rows( mat.m_rows ), m_columns( mat.m_columns ), m_size( mat.m_size ),
  m_totalRows( mat.m_totalRows ), m_totalColumns( mat.m_totalColumns ),
  m_totalSize( mat.m_totalSize ), m_rank( mat.m_rank ),
  m_numCpus( mat.m_numCpus )
{
  // would memcpy be faster?
  m_data = new T[ m_size ];
  for ( int i = 0; i < m_size; ++i )
  {
    m_data[i] = mat.m_data[i];
  }
}

template <class T>
NICMatrix<T>::~NICMatrix()
{
  if ( 0 != m_data ) delete[] m_data;
}

template <class T>
void
NICMatrix<T>::zeroMatrix()
{ 
  constantMatrix( 0 );
}

template <class T>
void
NICMatrix<T>::constantMatrix( const T& value )
{
  for ( int i = 0; i < m_size; ++i )
  {
    m_data[i] = value;
  }
}

template <class T>
void
NICMatrix<T>::identityMatrix() 
{
  int location;

  zeroMatrix();
  for ( int i = 0; i < std::min( m_rows, m_columns ); ++i )
  {
    location = ( i * m_columns ) + i;
    m_data[ location ] = 1;
  }
  if ( m_rows != m_columns )
  {
    std::cerr << "WARNING: (matrix<T>::identity_matrix)-- Matrix is not square" 
      << std::endl;
  }
}


// UPDATE THIS
// Casting everything to type T sucks. Should probably use RTTI to
// determine if complex and, if so, size *=2, cast to type of 
// complex and preceed
template <class T>
void
NICMatrix<T>::randomMatrix( double min, double max ) 
{
  // TODO implement me
/*  T * d = data;
  double delta = max - min;

  for (int i = 0; i < size;  i++) {
    *d++ = (T)(min + delta * rand()/((double)RAND_MAX));
  }
  */
}


// add the vector to the specified row
// WARNING! no bounds checking
template <class T>
void 
NICMatrix<T>::addRow( NICVector<T>& vec, int row ) 
{
  T* d = m_data + ( row * m_columns );
  T* v = vec.data();

  for (int i = 0; i < m_columns; ++i) 
  {
    *d = *v;
    ++d;
    ++v;
  }
}

// add the vector to the specified column
// WARNING! no bounds checking!
template <class T>
void
NICMatrix<T>::addColumn( NICVector<T>& vec, int column ) 
{
  T* d = m_data + column;
  T* v = vec.data();

  for ( int i = 0; i < m_rows; ++i ) 
  {
    *d = *v;
    d += m_columns;
    ++v;
  }
}

// WARNING! no bounds checking
template <class T>
void 
NICMatrix<T>::addDiagonal( NICVector<T>& vec ) 
{
  int location;
  T* v = vec.data();

  for ( int i = 0; i < std::min( m_rows, m_columns ); ++i )
  {
    location = ( i * m_columns ) + i;
    m_data[i] = *v;
    ++v;
  }
}

template <class T>
void
NICMatrix<T>::swapMatrices( NICMatrix<T>& mat )
{
  std::swap( m_rows, mat.m_rows );
  std::swap( m_columns, mat.m_columns );
  std::swap( m_size, mat.m_size );
  std::swap( m_totalRows, mat.m_totalRows );
  std::swap( m_totalColumns, mat.m_totalColumns );
  std::swap( m_totalSize, mat.m_totalSize );
  std::swap( m_data, mat.m_data );
}

template <class T>
NICMatrix<T>&
NICMatrix<T>::operator=( const NICMatrix<T>& rhs )
{
  if ( this == &rhs ) return *this;
  m_rows = rhs.m_rows;
  m_columns = rhs.m_columns;
  m_totalRows = rhs.m_totalRows;
  m_totalColumns = rhs.m_totalColumns;
  m_totalSize = rhs.m_totalSize;
  if ( m_size < rhs.m_size )
  {
    delete m_data;
    m_data = new T[ rhs.m_size ];
  }
  m_size = rhs.m_size;
  for ( int i = 0; i < m_size; ++i )
  {
    m_data[i] = rhs.m_data[i];
  }
  return *this;
}

template <class T>
T&
NICMatrix<T>::operator()( int row, int column )
{
  return m_data[ column + ( row * m_columns ) ];
}

template <class T>
std::ostream& operator<<( std::ostream& os, const NICMatrix<T>& v )
{
  for ( int i = 0; i < v.m_size; ++i )
  {
    os << v.m_data[i] << " ";
  }
  return os;
}

template <class T>
bool
NICMatrix<T>::read( std::istream& is, int rows, int columns )
{
  size_t sizeOfT = sizeof( T );
  char* nextValue = new char[ sizeOfT ];
  T* nextValueT;
  int numberToRead = rows * columns;

  if ( m_size < numberToRead )
  {
    delete [] m_data;
    m_data = new T[ numberToRead ];
  }

  m_size = numberToRead;
  m_rows = rows;
  m_columns = columns;
  int count = 0;

  while ( ( count < numberToRead ) &&
          ( is.read( nextValue, sizeOfT ) ).good() )
  {
    nextValueT = (T*)(nextValue);
    m_data[ count ] = *nextValueT;
    ++count;
  }

  delete [] nextValue;
  return ( count == numberToRead );
}

template <class T>
bool
NICMatrix<T>::readTransposed( std::istream& is, int rows, int columns )
{
    size_t sizeOfT = sizeof( T );
    char* nextValue = new char[ sizeOfT ];
    T nextValueT;
    int numberToRead = rows * columns;

    if ( m_size < numberToRead )
    {
        delete [] m_data;
        m_data = new T[ numberToRead ];
    }

    m_size = numberToRead;
    // think about this... the rows we pass in are the data file rows, not the
    // rows in this matrix
    m_rows = columns;
    m_columns = rows;

    for ( int i = 0; i < rows; ++i )
    {
        for ( int j = 0; j < columns; ++j )
        {
            is.read( nextValue, sizeOfT );
            nextValueT = *((T*)(nextValue));
            (*this)( j, i ) = nextValueT;
        }
    }

    delete [] nextValue;
    return true;
}

template <class T>
bool
NICMatrix<T>::write( std::ostream& os )
{
  size_t sizeOfT = sizeof(T);
  char* nextValue;

  for ( int i = 0; i < m_size; ++i )
  {
    nextValue = (char*)( m_data + i );
    os.write( nextValue, sizeOfT );
  }
  return true;
}

template <class T>
bool
NICMatrix<T>::writeTransposed( std::ostream& os )
{
  size_t sizeOfT = sizeof(T);
  T temp;
  char* nextValue = (char*)(&temp);

  for ( int i = 0; i < m_columns; ++i )
  {
      for ( int j = 0; j < m_rows; ++j )
      {
        temp = (*this)(j,i);
        os.write( nextValue, sizeOfT );
      }
  }
  return true;
}

template <class T>
bool
NICMatrix<T>::operator==( const NICMatrix<T>& rhs )
{
    if ( this == &rhs) return true;
    if ( this->m_rows != rhs.m_rows ||
         this->m_columns != rhs.m_columns )
    {
        return false;
    }

    for ( int i = 0; i < m_size; ++i )
    {
        if ( m_data[i] != rhs.m_data[i] ) return false;
    }
    return true;
}
    
template
std::ostream& operator<< <>( std::ostream& out, const NICMatrix<double>& m );
template
std::ostream& operator<< <>( std::ostream& out, const NICMatrix<float>& m );
template
std::ostream& operator<< <>( std::ostream& out, const NICMatrix<int>& m );

template class NICMatrix<double>;
template class NICMatrix<float>;
template class NICMatrix<int>;

