#ifndef NICMATRIX_H_UNIVERSITY_OF_OREGON_NIC
#define NICMATRIX_H_UNIVERSITY_OF_OREGON_NIC

#include <cstdlib>
#include <iostream>

// yet another refactoring and incarnation of the nefarious
// NICMatrix class. It might make sense to try to replace the
// HiPerSAT implementation with this one, which is arguably 
// cleaner
template <class T> class NICVector;
template <class T> class NICMatrix;

template <class T>
std::ostream& operator<<( std::ostream& os, const NICMatrix<T>& mat );

template <class T>
class NICMatrix 
{
public:
  // contructors and destructor
  NICMatrix(int rows = 0, int cols = 0, int totalRows = 0, int totalCols = 0, 
            int rank = 0, int numCpus = 1);
  NICMatrix( const NICMatrix<T>& mat );
  ~NICMatrix();

  // matrix initialization methods
  void zeroMatrix();
  void constantMatrix( const T& value );
  void identityMatrix();
  void randomMatrix( double min = 0, double max = 1 );

  // vector operations
  void addRow( NICVector<T>& vec, int row );
  void addColumn( NICVector<T>& vec, int col );
  void addDiagonal( NICVector<T>& data );

  // matrix operations
  void swapMatrices( NICMatrix<T>& mat );
  NICMatrix<T>& operator=( const NICMatrix<T> & rhs );

  // access methods
  T& operator()( int row, int column );
  T* data() { return m_data; }

  // io methods
  friend std::ostream& operator<< <>( std::ostream& os, const NICMatrix<T>& v );
  bool read( std::istream& is, int rows, int columns );
  bool readTransposed( std::istream& is, int rows, int columns );
  bool write( std::ostream& os );
  bool writeTransposed( std::ostream& os );

  // comparison
  bool operator==( const NICMatrix<T>& rhs );
  bool operator!=( const NICMatrix<T>& rhs ) { return !( *this == rhs ); }

private:
  int m_rows;
  int m_columns;
  int m_size;

  int m_totalRows;
  int m_totalColumns;
  int m_totalSize;

  int m_rank;
  int m_numCpus;

  T* m_data;

};

#endif
// NICMATRIX_H_UNIVERSITY_OF_OREGON_NIC
