#ifndef MATRIXREADER_CPP_UNIVERSITY_OF_OREGON_NIC
#define MATRIXREADER_CPP_UNIVERSITY_OF_OREGON_NIC

#include <fstream>
#include "DataReader.h"
#include "MatrixReader.h"
#include "NicMatrix.h"

using namespace std;

template <class T>
bool MatrixReader<T>::loadMatrix( 
    NicMatrix<T>& matrix, 
    DataFormat::DataFormat format, 
    int rows,
    int columns,
    const string& fileName )
{
    // set the mode for file io
    ios::openmode mode = ios::in;
    if ( format != DataFormat::TEXT ) mode |= ios::binary;
    typename DataReader<T>::Endian endian;
    typename DataReader<T>::Format readerFormat;

    switch ( format )
    {
        case DataFormat::BIG:
            endian = DataReader<T>::BigEndian;
            readerFormat = DataReader<T>::Binary;
            break;
        case DataFormat::LITTLE:
            endian = DataReader<T>::LittleEndian;
            readerFormat = DataReader<T>::Binary;
            break;
        case DataFormat::NATIVE:
            endian = DataReader<T>::NativeEndian;
            readerFormat = DataReader<T>::Binary;
            break;
        case DataFormat::TEXT:
            endian = DataReader<T>::NativeEndian;
            readerFormat = DataReader<T>::Ascii;
            break;
        default:
            endian = DataReader<T>::NativeEndian;
            readerFormat = DataReader<T>::Ascii;
    }

    ifstream inputFile( fileName.c_str(), mode );
    if ( !inputFile.good() )
    {
        return false;
    }

    matrix.resize( rows, columns );

    DataReader<T> reader( inputFile, readerFormat, endian );
    
    T* d = matrix.data;
    int count = 0;
    int numValues = rows * columns;
    while ( (count < numValues) && reader.read( d ) )
    {
        ++d;
        ++count;
    }

    inputFile.close();
    
    return ( count == numValues );
}

#ifdef INSTANTIATE_TEMPLATES
template class MatrixReader<double>;
template class MatrixReader<float>;
#endif

#endif
// MATRIXREADER_CPP_UNIVERSITY_OF_OREGON_NIC
